#ifndef __HARDWARE_H__
#define __HARDWARE_H__

#include <pic18f252.h>
#include <stdbool.h>

/* IO defines*/
#define	PROG_B		TRISAbits.TRISA2	/* FPGA config */
#define	PROG_B_VAL	PORTAbits.RA2		/* FPGA config */
#define	DONE		PORTBbits.RB3		/* FPGA config */
#define	INIT_B		PORTAbits.RA3		/* FPGA config */
#define	CCLK		LATAbits.LATA5		/* FPGA config */
#define	DIN		LATBbits.LATB7		/* FPGA config */
#define	_SW0		PORTBbits.RB0		/* Switch input */
#define	_SW1		PORTBbits.RB1		/* Switch input */
#define	_M_CD		TRISAbits.TRISA0	/* mmc card clock disable */
#define	_M_CS		PORTCbits.RC0		/* mmc card spi select */
#define	_F_CS0		PORTAbits.RA1		/* FGPA spi0 select */
#define	SCSI_HSHAKE	PORTBbits.RB5		/* SCSI data request */
#define	NMI		PORTBbits.RB6		/* Programmer switch */

/* Some IO accessors. static inline somewhat broken with sdcc so
 * I use macros... sad
 */
#define spi_enable_fpga() do {		\
	_F_CS0 = 0;			\
} while(0)

#define spi_disable_fpga() do {		\
	_F_CS0 = 1;			\
} while(0)

#define spi_enable_osd() do {		\
	_F_CS1 = 0;			\
} while(0)

#define spi_disable_osd() do {		\
	_F_CS1 = 1;			\
} while(0)

#define spi_enable_sdcard() do {	\
	_M_CD = 1;			\
	_M_CS = 0;			\
} while(0)

#define spi_disable_sdcard() do {	\
	_M_CS = 1;			\
	do_spi(0xff);			\
	_M_CD = 0;			\
} while(0)

#define check_button(void) (!_SW0)

#define reset_fpga() do {		\
	PROG_B_VAL = 0;			\
	PROG_B = 0;			\
	PROG_B = 1;			\
} while(0)

#define diskled_on() do {		\
	PORTBbits.RB4 = 1;		\
} while(0)

#define diskled_off() do {		\
	PORTBbits.RB4 = 0;		\
} while(0)

/* Functions */
extern void hardware_init(void);

extern unsigned short get_timer(unsigned short offset);
extern char check_timer(unsigned short t);
extern void wait_timer(unsigned short time);

extern unsigned char do_spi(unsigned char data);
extern void shift_fpga(unsigned char data);

/* Backbus interface defines */
#define BB_WRITE		0x80
#define BB_AUTOINC		0x40

#define BB_REG_CTRL_BASE        0x00
#define BB_REG_CPU_BASE		0x10
#define BB_REG_MEM_BASE		0x20
#define BB_REG_IWM_BASE		0x28
#define BB_REG_SCSI_BASE	0x30
#define BB_REG_SCOPE_BASE	0x38


#endif /* __HARDWARE_H__ */

