#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "hardware.h"
#include "mmc.h"
#include "fat16.h"
#include "scsi.h"

#define printf	printf_tiny

/* Sector buffer. Shared globally to save space on PIC
 *
 * XXX In this file to work around a missing bank select problem
 */
unsigned char secbuf[512];

static char hex[] = {'0','1','2','3','4','5','6','7','8','9','a','b','c',
		     'd','e','f'};

/* Temp buffer used for tests */
static unsigned char c[10];

static unsigned char load_file(char *name, unsigned long addr)
{
	unsigned short t;
	unsigned short n;
	unsigned char *ptr = secbuf;
	long size;

	size = fat_open(name);
	if (size < 0) {
		printf("File %s not found !\n", name);
		return false;
	}
	fat_file_seek(0);

	spi_enable_fpga();
	do_spi(BB_WRITE | BB_AUTOINC | BB_REG_MEM_BASE);
	do_spi((addr >> 16) & 0xff);
	do_spi((addr >>  8) & 0xff);
	do_spi((addr      ) & 0xff);
	spi_disable_fpga();

	t = 0;
	n = (size + 7) >> 3;
	do {
		/* read sector if 512 (64*8) bytes done */
		if ((t & 0x3f) == 0) {
			if ((t>>9)&1)
				diskled_on();
			else
				diskled_off();
			putchar('*');
			if (!fat_file_read()) {
				printf("Reading failed !\n");
				return false;
			}
			spi_enable_fpga();
			do_spi(BB_WRITE | BB_REG_MEM_BASE | 3);
			ptr = secbuf;
		}

		/* send data in packets of 8 bytes */
		do_spi(*(ptr++));
		do_spi(*(ptr++));
		do_spi(*(ptr++));
		do_spi(*(ptr++));
		do_spi(*(ptr++));
		do_spi(*(ptr++));
		do_spi(*(ptr++));
		do_spi(*(ptr++));
		t++;

		/* read next sector if 512 (64*8) bytes done */
		if ((t & 0x3f) == 0) {
			spi_disable_fpga();
			fat_file_next_sector();
		}
		if ((t & 0x7ff) == 0)
			putchar(10);
	} while (t < n);
	spi_disable_fpga();

	putchar(10);

	return true;
}

static unsigned char configure_fpga(void)
{
	unsigned short t;
	unsigned short n;
	unsigned char *ptr = secbuf;
	long size;

	/* Reset FGPA configuration sequence */
	reset_fpga();

	// now wait for INIT to go high
	t = 50000;
	while (!INIT_B) {
		if (--t==0) {
			printf("FPGA init is NOT high!\n");
			return false;
		}
	}
	if (DONE) {
		printf("FPGA done is high before configuration!\n");
		return false;
	}

	/* Open bitstream file */
	size = fat_open("MIMIGMACBIN");
	if (size < 0) {
		printf("No FPGA configuration file found!\n");
		return false;
	}
	fat_file_seek(0);

	/* Send all bytes to FPGA in loop */
	t = 0;
	n = size >> 3;
	do {
		/* read sector if 512 (64*8) bytes done */
		if ((t & 0x3f) == 0) {
			if ((t>>9)&1)
				diskled_on();
			else
				diskled_off();
			putchar('*');
			if (!fat_file_read()) {
				printf("Reading failed !\n");
				return false;
			}
			ptr = secbuf;
		}

		/* send data in packets of 8 bytes */
		shift_fpga(*(ptr++));
		shift_fpga(*(ptr++));
		shift_fpga(*(ptr++));
		shift_fpga(*(ptr++));
		shift_fpga(*(ptr++));
		shift_fpga(*(ptr++));
		shift_fpga(*(ptr++));
		shift_fpga(*(ptr++));
		t++;

		/* read next sector if 512 (64*8) bytes done */
		if ((t & 0x3f) == 0)
			fat_file_next_sector();
		if ((t & 0x7ff) == 0)
			putchar(10);
	} while (t < n);

	shift_fpga(0xff);
	shift_fpga(0xff);
	shift_fpga(0xff);
	shift_fpga(0xff);
	shift_fpga(0xff);
	shift_fpga(0xff);
	shift_fpga(0xff);
	shift_fpga(0xff);
	shift_fpga(0xff);
	shift_fpga(0xff);

	diskled_off();

	/* check if DONE is high */
	if (DONE) {
		shift_fpga(0xff);
		shift_fpga(0xff);
		shift_fpga(0xff);
		shift_fpga(0xff);
		shift_fpga(0xff);
		shift_fpga(0xff);
		shift_fpga(0xff);
		shift_fpga(0xff);
		shift_fpga(0xff);
		shift_fpga(0xff);
		putchar(10);
		return true;
	}

	printf("FPGA done is NOT high!\n");
	return false;
}

static void print_latches(void)
{
	printf("  RW:%d U:%d L:%d ADDR: %x %x %x DATA: %x %x\n",
	       !!(c[5] & 0x01),
	       !!(c[5] & 0x02),
	       !!(c[5] & 0x04),
	       c[0], c[1], c[2], c[3], c[4]);
}

#if 1
static void dump_scope(void)
{
	unsigned short i;

	/* Dump scope output */
	spi_enable_fpga();
	do_spi(BB_AUTOINC | BB_REG_SCOPE_BASE | 2);
	do_spi(0xff);
	c[0] = do_spi(0xff);
	c[1] = do_spi(0xff);
	printf("sample count: %x %x\n", c[0], c[1]);
	spi_disable_fpga();
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_REG_SCOPE_BASE | 4);
	do_spi(0xff);
	spi_disable_fpga();

	spi_enable_fpga();
	do_spi(BB_REG_SCOPE_BASE | 4);
	do_spi(0xff);
	//	for (i = 0; i < 0x200; i++) {
	for (i = 0; i < (32768/8); i++) {
		c[0] = do_spi(0xff);
		c[1] = do_spi(0xff);
		c[2] = do_spi(0xff);
		c[3] = do_spi(0xff);
		c[4] = do_spi(0xff);
		c[5] = do_spi(0xff);
		c[6] = do_spi(0xff);
		c[7] = do_spi(0xff);
		printf("%x: %x %x %x %x %x %x %x %x\n", i * 8,
		       c[0], c[1], c[2], c[3],
		       c[4], c[5], c[6], c[7]);
	}
	spi_disable_fpga();
}
#endif

static unsigned char check_fpga(void)
{
	spi_enable_fpga();
	do_spi(BB_AUTOINC | BB_REG_CTRL_BASE);
	do_spi(0xff);	/* pad */
	c[0] = do_spi(0xff);	/* Read data 0 : ID */
	c[1] = do_spi(0xff);    /* Read data 1 : Version */
	spi_disable_fpga();

	printf("FPGA ID %x Version %d\n", c[0], c[1]);
	if (c[0] != 0x42 && c[1] != 1) {
		printf("Unsupported !\n");
		return false;
	}

	/* Test CPU inteface latches */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_AUTOINC | BB_REG_CPU_BASE);
	do_spi(0xde);
	do_spi(0xad);
	do_spi(0xbf);
	do_spi(0xf0);
	do_spi(0x0d);
	do_spi(0xff);
	spi_disable_fpga();

	spi_enable_fpga();
	do_spi(BB_AUTOINC | BB_REG_CPU_BASE);
	do_spi(0xff);
	c[0] = do_spi(0xff);
	c[1] = do_spi(0xff);
	c[2] = do_spi(0xff);
	c[3] = do_spi(0xff);
	c[4] = do_spi(0xff);
	c[5] = do_spi(0xff);
	spi_disable_fpga();

	if (c[0] != 0xde || c[1] != 0xad || c[2] != 0xbe ||
	    c[3] != 0xf0 || c[4] != 0x0d) {
		printf("Latches test FAILED want:\n");		
		printf("  RW:1 U:1 L:1 ADDR: de ad be DATA: f0 0d\n");
		printf("Got:\n");
		print_latches();
		return false;
	}
	printf("CPU interface latch test passed !\n");

	return true;
}

/* Bits inverted from what goes into the SPI request */
#define SIM_CYCLE_READ	0x00
#define SIM_CYCLE_WRITE	0x01
#define SIM_CYCLE_UPPER	0x02
#define SIM_CYCLE_LOWER	0x04
#define SIM_CYCLE_WORD	(SIM_CYCLE_UPPER | SIM_CYCLE_LOWER)

static unsigned short sim_bus_cycle(unsigned long addr,
				    unsigned short dat,
				    unsigned char flags)
{
	unsigned char stat;

	/* Read value at 0x1020 */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_AUTOINC | BB_REG_CPU_BASE);
	do_spi((addr >> 16) & 0xff);
	do_spi((addr >>  8) & 0xff);
	do_spi((addr      ) & 0xff);
	do_spi((dat  >> 8 ) & 0xff);
	do_spi((dat       ) & 0xff);
	do_spi(~flags);
	do_spi(0x04); /* ctrl: cycle */
	spi_disable_fpga();

	/* Wait for stop state */
	spi_enable_fpga();
	do_spi(BB_REG_CPU_BASE + 0x7);
	do_spi(0xff);
	do
		stat = do_spi(0xff);
	while(!(stat & 0x08));
	spi_disable_fpga();

	/* Read latches */
	spi_enable_fpga();
	do_spi(BB_AUTOINC | BB_REG_CPU_BASE | 3);
	do_spi(0xff);
	dat  = do_spi(0xff);
	dat  <<= 8;
	dat  |= do_spi(0xff);
	spi_disable_fpga();

	return dat ;
}

static void dump_latches(void)
{
	/* Read latches */
	spi_enable_fpga();
	do_spi(BB_AUTOINC | BB_REG_CPU_BASE);
	do_spi(0xff);
	c[0] = do_spi(0xff);
	c[1] = do_spi(0xff);
	c[2] = do_spi(0xff);
	c[3] = do_spi(0xff);
	c[4] = do_spi(0xff);
	c[5] = do_spi(0xff);
	spi_disable_fpga();
	printf("  RW:%d U:%d L:%d ADDR: %x %x %x DATA: %x %x %s\n",
	       !!(c[5] & 0x01),
	       !!(c[5] & 0x02),
	       !!(c[5] & 0x04),
	       c[0], c[1], c[2], c[3], c[4],
	       SCSI_HSHAKE ? " [SCSI]" : "");
}

static void run_to(unsigned long addr)
{
	unsigned char stat;

	/* Set breakpoint */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_AUTOINC | BB_REG_CPU_BASE | 8);
	do_spi((addr >> 16) & 0xff);
	do_spi((addr >>  8) & 0xff);
	do_spi((addr      ) & 0xff);
	spi_disable_fpga();

	/* Start CPU with BP */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_REG_CPU_BASE | 6);
	do_spi(0x42); /* RUN + BKEN */
	spi_disable_fpga();

	/* Wait for stop state */
	spi_enable_fpga();
	do_spi(BB_REG_CPU_BASE + 0x7);
	do_spi(0xff);
	do {
		if (SCSI_HSHAKE) {
			spi_disable_fpga();
			do_scsi();
			spi_enable_fpga();
			do_spi(BB_REG_CPU_BASE + 0x7);
			do_spi(0xff);
		}
		stat = do_spi(0xff);
	} while(!(stat & 0x08));
	spi_disable_fpga();
}

static void step(unsigned short n)
{
	unsigned short i;

	for (i = 0; i < n; i++) {
		/* Step CPU */
		spi_enable_fpga();
		do_spi(BB_WRITE | BB_REG_CPU_BASE | 6);
		do_spi(0x82); /* RUN + STEP */
		spi_disable_fpga();

#if 0 /* Need in theory but CPU never fast enough */
		/* Wait for stop state */
		spi_enable_fpga();
		do_spi(BB_REG_CPU_BASE + 0x7);
		do_spi(0xff);
		do
			stat = do_spi(0xff);
		while(!(stat & 0x08));
		spi_disable_fpga();
#endif
	}
}

void main(void)
{
	unsigned char rc;
	unsigned short i;
	unsigned short v, v2;

	hardware_init();

	printf("Initializing MMC...\n");

	rc = mmc_init();
	if (!rc)
		return;

	printf("Initializing FAT...\n");

	rc = fat_init();
	if (!rc)
		return;

	printf("Configuring FPGA...\n");

	if (!configure_fpga())
		return;

	//	printf("Checking FPGA...\n");
	if (!check_fpga())
		return;

	printf("Loading ROM file...\n");
	if (!load_file("MIMIGMACROM", 0x200000))
		return;
#if 0
	printf("Loading Screenshot...\n");
	if (!load_file("SCRNSHOTRAW", 0x1fa700))
		return;
#endif

	printf("Opening SCSI disk image...\n");
	scsi_open();

#if 0
	/* Start scope */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_REG_SCOPE_BASE);
	do_spi(0x02);
	spi_disable_fpga();

	/* Read ROM first words */
	v = sim_bus_cycle(0, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	v2 = sim_bus_cycle(2, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("Mem 0: %x %x\n", v, v2);
	v = sim_bus_cycle(4, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	v2 = sim_bus_cycle(6, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("Mem 4: %x %x\n", v, v2);

	/* Read ROM las words */
	v = sim_bus_cycle(0x41fff8, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	v2 = sim_bus_cycle(0x41fffa, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("ROM end 0: %x %x\n", v, v2);
	v = sim_bus_cycle(0x41fffc, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	v2 = sim_bus_cycle(0x41fffe, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("ROM end 4: %x %x\n", v, v2);
#endif

	/* Un-reset CPU */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_REG_CTRL_BASE | 2);
	do_spi(0x01);
	spi_disable_fpga();

#if 0
	{
		unsigned char b,oldb,t;

		spi_enable_fpga();
		do_spi(BB_REG_CTRL_BASE | 6);
		do_spi(0xff);
		oldb = 0;
		t = 0, i = 0;
		while (i < 512) {
			oldb = b;
			b = do_spi(0xff);
			if (b == oldb && t < 0xff)
			    t++;
			else {
				secbuf[i++] = b;
				secbuf[i++] = t;
				t = 0;
			}
		}
		spi_disable_fpga();
		for (i = 0; i < 512; i+=2)
			printf("%x %x\n", secbuf[i], secbuf[i+1]);
	}
#endif

#if 0


	//	printf("Running to main()\n");
	//	run_to(0x4010c8);
	printf("Stepping...()\n");
	dump_latches();
	step(1);
	dump_latches();
	step(1);

	for (i = 0; i < 1000; i++) {
		step(1);
		dump_latches();
	}

	for (;;)
		;
#endif
#if 0
	for (;;) {
		printf("Running to exception \n");
		run_to(0x401336);
		dump_latches();
		step(1);
	}
#endif

#if 0
	printf("Running to P_mBootBeep...\n");
	run_to(0x40028a);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to IWM init...\n");
	run_to(0x4000fc);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to P_ChecksumRomAndTestMemory...\n");
	run_to(0x400d76);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to P_BootPart2 (fake hit)...\n");
	run_to(0x400352);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to P_BootPart2...\n");
	run_to(0x400352);
	dump_latches();
	step(1);
	dump_latches();
#endif

#if 0
	/* Read video first words */
	v = sim_bus_cycle(0x1fa700, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	v2 = sim_bus_cycle(0x1fa702, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("Vid 0: %x %x\n", v, v2);
	v = sim_bus_cycle(0x1fa704, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	v2 = sim_bus_cycle(0x1fa706, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("Vid 4: %x %x\n", v, v2);
	printf("Running to setting MemTop...\n");
	run_to(0x40037e);
	dump_latches();
	step(1);
	dump_latches();
	step(1);
	dump_latches();
	step(1);
	dump_latches();
	step(1);
	dump_latches();
	v = sim_bus_cycle(0x108, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("Detected memory size: 0x%x0000 bytes\n", v);
	printf("Running to L53 (CPUflag test)...\n");
	run_to(0x400584);
	dump_latches();
	for (i = 0; i < 50; i++) {
		step(1);
		dump_latches();
	}
	printf("Running to Bsr P5...\n");
	run_to(0x4005fa);
	dump_latches();
	printf("Running to call to _InitResources...\n");
	run_to(0x400600);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to P_mInitQueue...\n");
	run_to(0x401a38);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to P_InitSCC...\n");
	run_to(0x40082c);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to P_InitKeyboard...\n");
	run_to(0x402568);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to P_mInitIOMgr...\n");
	run_to(0x40087c);
	dump_latches();
	step(1);
	dump_latches();
#endif
#if 0

	printf("Running to P252...\n");
	run_to(0x407d40);
	dump_latches();

	/* HWCfgFlags at 0xb22 bit 7 is SCSI */
	v = sim_bus_cycle(0xb22, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("HWCfgFlags=%x\n", v >> 8);
	v = sim_bus_cycle(0xb2e, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("SCSIDrvrs =%x\n", v >> 8);

	printf("Running to SCSI driver jump...\n");
	run_to(0x407dc4);
	dump_latches();
	printf("Running to SCSI driver csum...\n");
	run_to(0x1d36);
	dump_latches();
#endif

#if 0
	printf("Running to after plot disk...\n");
	run_to(0x4006e4);
	dump_latches();
	v = sim_bus_cycle(0x30a, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	v2 = sim_bus_cycle(0x30c, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("DrvQHdr.Head=%x %x\n", v, v2);
#endif

#if 0
	printf("Running to P_mInitIOMgr..x1...\n");
	run_to(0x40089c);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to P_mInitIOMgr..x2...\n");
	run_to(0x4008a6);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to P_mInitIOMgr..x3...\n");
	run_to(0x4008ac);
	dump_latches();
#endif

#if 0
	for (i = 0; i < 1000; i++) {
		step(1);
		dump_latches();
	}
#endif
	/* Run CPU */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_REG_CPU_BASE | 6);
	do_spi(0x02);
	spi_disable_fpga();

	i = 0;
	for (;;) {
		/* This is VERY timing sensitive, the ROM selection
		 * timeout is very short and the fact that we run at
		 * twice the speed doesn't help.
		 * We could improve that by pre-setting the SPI to point
		 * to the SCSI lines register and just read from them,
		 * or we could have some of the selection logic moved to
		 * the FPGA
		 *
		 * Better HACK: The FPGA could delay CPU read cycles
		 *              from the bus status register when SEL
		 *              is set by a significant amount in the
		 *              FPGA thus allowing a lot more time for
		 *              the selection process
		 */
		if (SCSI_HSHAKE)
			do_scsi();
	}

#if 0
	printf("Running to E_Sony_Open...\n");
	run_to(0x417d9e);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to E_Sony_Open:17DFA (_VInstall)...\n");
	run_to(0x417dfa);
	dump_latches();
	step(1);
	dump_latches();
	printf("Running to E_Sony_Open:17E12 (bsr P721)...\n");
	run_to(0x417e12);
	dump_latches();
	for (i = 0; i < 1000; i++) {
		step(1);
		dump_latches();
	}
	printf("Running to E_Sony_Open:17E16 (bsr P720)...\n");
	run_to(0x417e12);
	dump_latches();
	for (i = 0; i < 100; i++) {
		step(1);
		dump_latches();
	}
	printf("Running to E_Sony_Open:17E24 (bsr P715)...\n");
	run_to(0x417e12);
	dump_latches();
	step(1);
	dump_latches();
	step(1);
	dump_latches();
	for (i = 0; i < 1000; i++) {
		step(1);
		dump_latches();
	}
	printf("Running to L58...\n");
	run_to(0x400692);
	dump_latches();
	step(1);
	dump_latches();
	for (i = 0; i < 1000; i++) {
		step(1);
		dump_latches();
	}
#endif


#if 0
	for (i = 0; i < 100; i++) {
		wait_timer(100);
		spi_enable_fpga();
		do_spi(BB_WRITE | BB_REG_CPU_BASE | 6);
		do_spi(0x01); /* RUN + STEP */
		spi_disable_fpga();
	
		/* We shall wait for cycle to complete but we know
		 * how slow our SPI is ...
		 */
		dump_latches();

		spi_enable_fpga();
		do_spi(BB_WRITE | BB_REG_CPU_BASE | 6);
		do_spi(0x02); /* RUN + STEP */
		spi_disable_fpga();
	}

	/* Stop CPU */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_REG_CPU_BASE | 6);
	do_spi(0x01); /* RUN + STEP */
	spi_disable_fpga();
#endif

#if 0
	step(10000);
	step(10000);
	step(10000);
	step(10000);
	step(10000);
	step(10000);
	step(10000);
	step(10000);
	step(10000);
	step(10000);
	step(10000);
#endif

#if 0
	/* Stop scope */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_REG_SCOPE_BASE);
	do_spi(0x00);
	spi_disable_fpga();

	printf("Reading 0x1020 via SPI initiated read cycle: ");
	v = sim_bus_cycle(0x1020, 0, SIM_CYCLE_READ | SIM_CYCLE_WORD);
	printf("%x\n", v);

	/* Reading memory via RAM fifo */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_AUTOINC | BB_REG_MEM_BASE);
	do_spi(0x20); /* ROM base */
	do_spi(0x10);
	do_spi(0x00);
	spi_disable_fpga();

	spi_enable_fpga();
	do_spi(BB_REG_MEM_BASE);
	do_spi(0xff);
	do_spi(0xff); /* one more dummy cycle with RAM fifo */
	for (i = 0; i < 0x40; i++) {
		printf(" %x", do_spi(0xff));
		if ((i & 0xf) == 0xf)
			printf("\n");
	}
	printf("\n");
#endif

#if 0
	/* Start CPU */
	spi_enable_fpga();
	do_spi(BB_WRITE | BB_REG_CPU_BASE | 6);
	do_spi(0x02); /* RUN */
	spi_disable_fpga();
#endif

#if 0
	for (;;)
		;
#endif
}
